/* Copyright 2019 Andrei-Lucian Birziche */
#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "mpu6050/mpu6050.h"
#include <math.h>
double ax = 0, ay = 0, az = 0, gx = 0, gy = 0, gz = 0;

void pwm_init(void) {
    //fast pwm cu top la oxff
	TCCR0A |= (1 << COM0A1) | (1 << COM0B1) | (1 << WGM01) | (1 << WGM00);
	// prescaler 1
	TCCR0B |= (1 << CS00);
	// fara intreruperi de match sau overflow
	TIMSK0 = 0;

	OCR0A = 0;
	OCR0B = 0;
}
	

void ports_init(void) {
	PORTD = 0;
	PORTB = 0;
	DDRB = 0;
	DDRD = 0;

	// input for motor driver
	DDRD |= (1 << PD0 | 1 << PD1);
	DDRB |= (1 << PB0 | 1 << PB1);
	
	// pwm pins
	DDRB |= (1 << PB3) | (1 << PB4);

	//stdby
	DDRD |= (1 << PD6);
	PORTD |= (1 << PD6);

	DDRC = 0;
	PORTC = 255;
}

void drive(float spdA, float spdB) {
	/* Prima roata; viteza negativa, merge in spate */
	if (spdA < 0) {
		PORTB |= (1 << PB0);
		PORTB &= ~(1 << PB1);
		OCR0A = -spdA;
	/* Prima roata; viteza pozitiva, merge in fata */
	} else {
		PORTB &= ~(1 << PB0);
		PORTB |= (1 << PB1);
		OCR0A = spdA;
	}
	/* A doua roata; viteza negativa, merge in spate */
	if (spdB < 0) {
		PORTD |= (1 << PD1);
		PORTD &= ~(1 << PD0);
		OCR0B = -spdB;
	/* A doua roata; viteza pozitiva, merge in fata */
	} else {
		PORTD &= ~(1 << PD1);
		PORTD |= (1 << PD0);
		OCR0B = spdB;
	}
}

int main() {
	float output, ki, kp, kd, p = 0, i = 0, d = 0, lastAngle = 0;
	float angle = 0, dt = 0.01;
	float gzF = 0; // viteza de rotatie filtrata
	
	ports_init();
	pwm_init();
	mpu6050_init();

	kp = 50;
	ki = 10;
	kd = 300;

	while(1) {

		mpu6050_getConvData(&ax, &ay, &az, &gx, &gy, &gz);

		gzF = (gy/16.4) * 0.1 + gzF * 0.9;

		/* obtin unghiul de inclinare al robotului integrand viteza de rotatie */
		lastAngle = angle;
		angle += gzF * dt;

		if (angle > 10)
			angle = 10;
		if (angle < -10)
			angle = -10;

		p = angle * kp;
		i = i + angle * ki;
		d = (angle - lastAngle) * kd;
		output = p + i + d;

		if(i < -100)
			i = -100;
		if(i > 100)
			i = 100;
		
		if(output < -255.0)
			output = -255.0;
		if(output > 255.0)
			output = 255.0;
		
		/* output negativ deoarece am conectat invers motoarele */
		drive(-output, -output);
	}

	return 0;
}

